/* The contents of this file are subject to the Mozilla Public
 * License Version 1.1 (the "License"); you may not use this file
 * except in compliance with the License. You may obtain a copy of
 * the License at http://www.mozilla.org/MPL/
 * 
 * Software distributed under the License is distributed on an "AS
 * IS" basis, WITHOUT WARRANTY OF ANY KIND, either express or
 * implied. See the License for the specific language governing
 * rights and limitations under the License.
 * 
 * The Original Code is Eric Hodel's <drbrain@segment7.net> code.
 * 
 * The Initial Developer of the Original Code is Eric Hodel.
 * Portions created by Eric Hodel are Copyright (C) 2001 Eric
 * Hodel. All Rights Reserved.
 * 
 * Other Contributors:
 *      Christopher Hoess <choess@force.stwing.upenn.edu>
 *      Tim Taylor <tim@tool-man.org>
 */


/** 
 * LinkToolbarHandler is a Singleton that displays LINK elements 
 * and nodeLists of LINK elements in the Link Toolbar.  It
 * associates the LINK with a corresponding LinkToolbarItem based 
 * on it's REL attribute and the toolbar item's ID attribute.
 * LinkToolbarHandler is also a Factory and will create 
 * LinkToolbarItems as necessary.
 */
function LinkToolbarHandler() {

  this.items = new Array();

  this.handleLinks = function(nodeList) {
    if (!nodeList) return;

    for (var i = 0; i < nodeList.length; i++) {
      // optimization: weed out useless links here to avoid
      // the function call. More in depth checks are done
      // in this.handle
      var node = nodeList.item(i);
      if (node && (node.rel || node.rev) && node.href) 
        this.handle(node);
    }
  }

  this.handle = function(element) {
    // XXX: if you're going to re-enable handling of anchor elements,
    //    you'll want to change this to AnchorElementDecorator
    var element = new LinkElementDecorator(element);

    if (element.isIgnored()) return;

    var relAttributes = element.rel.split(" ");
    for (var i = 0; i < relAttributes.length; i++) {
      var linkType = getLinkType(relAttributes[i]);
      this.getItemForLinkType(linkType).displayLink(element);
    }
  }

  function getLinkType(relAttribute) {
    switch (relAttribute.toLowerCase()) {
      case "home":
      case "start":
      case "top":
      case "origin":
        return "top";

      case "up":
      case "parent":
        return "up";

      case "begin":
      case "first":
        return "first";

      case "next":
      case "child":
        return "next";

      case "prev":
      case "previous":
        return "prev";

      case "end":
      case "last":
        return "last";

      case "author":
      case "made":
        return "author";

      case "contents":
      case "toc":
        return "toc";

      default:
        return relAttribute.toLowerCase();
    }
  }

  this.getItemForLinkType = function(linkType) {
    if (!this.items[linkType])
      this.items[linkType] = createItemForLinkType(linkType);

    return this.items[linkType];
  }

  function createItemForLinkType(linkType) {
    if (!document.getElementById("link-" + linkType))
      return new LinkToolbarTransientMenu(linkType);

    // XXX: replace switch with polymorphism
    switch(document.getElementById("link-" + linkType).tagName) {
      case "button":
        return new LinkToolbarButton(linkType);

      case "menuitem":
        return new LinkToolbarItem(linkType);

      case "menu":
        return new LinkToolbarMenu(linkType);

      default:
        return new LinkToolbarTransientMenu(linkType);
    }
  }

  this.clearAllItems = function() {
    for (var linkType in this.items)
      this.items[linkType].clear();
  }
}
const linkToolbarHandler = new LinkToolbarHandler();


function LinkElementDecorator(element) {
  /*
   * XXX: this is an incomplete decorator, because it doesn't implement
   *      the full Element interface.  If you need to use a method 
   *      or member in the Element interface, just add it here and
   *      have it delegate to this.element
   *
   * XXX: would rather add some methods to Element.prototype instead of
   *    using a decorator, but Element.prototype is no longer exposed 
   *      since the XPCDOM landing, see bug 83433
   */

  this.isIgnored = function() {
    if (!this.rel) return true;
    if (this.isStylesheet()) return true;
    if (this.isIcon()) return true;

    return false;
  }

  this.isStylesheet = function() {
    if (findWord("stylesheet", this.rel.toLowerCase()))
      return true;
    else
      return false;
  }

  this.isIcon = function() {
    if (findWord("icon", this.rel.toLowerCase()))
      return true;
    else
      return false;

    return false;
  }

  function convertRevMade(rel, rev) {
    if (!rel && rev && findWord("made", rev.toLowerCase()))
      return rev;
    else
      return rel;
  }

  this.getTooltip = function() {
    return this.getLongTitle() != "" ? this.getLongTitle() : this.href;
  }

  this.getLabel = function() {
    return this.getLongTitle() != "" ? this.getLongTitle() : this.rel;
  }

  this.getLongTitle = function() {
    if (this.longTitle == null)
      this.longTitle = this.makeLongTitle();

    return this.longTitle;
  }

  this.makeLongTitle = function() {
    var prefix = "";

    // XXX: lookup more meaningful and localized version of media, 
    //   i.e. media="print" becomes "Printable" or some such
    // XXX: use localized version of ":" seperator
    if (this.media 
        && !findWord("all", this.media.toLowerCase())
        && !findWord("screen", this.media.toLowerCase()))
      prefix += this.media + ": ";
    if (this.hreflang)
      prefix += languageDictionary.lookupLanguageName(this.hreflang)
            + ": ";

    return this.title ? prefix + this.title : prefix;
  }

  function findWord(word, string) {
    var barePattern = eval("/^" + word + "$/");
    var middlePattern = eval("/[-\\W]" + word + "[-\\W]/");
    var startPattern = eval("/^" + word + "[-\\W]/");
    var endPattern = eval("/[-\\W]" + word + "$/");

    if (string.search(barePattern) != -1)
      return true;
    if (string.search(middlePattern) != -1)
      return true;
    if (string.search(startPattern) != -1)
      return true;
    if (string.search(endPattern) != -1)
      return true;

    return false;
  }

  if (!element) return; // skip the rest on foo.prototype = new ThisClass calls

  this.element = element;

  this.rel = convertRevMade(element.rel, element.rev);
  this.rev = element.rev;
  this.title = element.title;
  this.href = element.href;
  this.hreflang = element.hreflang;
  this.media = element.media;
  this.longTitle = null;
}


function AnchorElementDecorator(element) {
  this.constructor(element);

  this.getLongTitle = function() {
    return this.title ? this.__proto__.getLongTitle.apply(this) 
        : getText(this.element);
  }

  function getText(element) {
    return condenseWhitespace(getTextRecursive(element));
  }

  function getTextRecursive(node) {
    var text = "";
    node.normalize();
    if (node.hasChildNodes()) {
      for (var i = 0; i < node.childNodes.length; i++) {
        if (node.childNodes.item(i).nodeType == Node.TEXT_NODE)
          text += node.childNodes.item(i).nodeValue;
        else if (node.childNodes.item(i).nodeType == Node.ELEMENT_NODE)
          text += getTextRecursive(node.childNodes.item(i));
      }
    }

    return text;
  }

  function condenseWhitespace(text) {
    return text.replace(/\W*$/, "").replace(/^\W*/, "").replace(/\W+/g, " ");
  }
}
AnchorElementDecorator.prototype = new LinkElementDecorator;
